@@ -77,7 +77,9 @@
 #define CODE_DOUBLE_ARRAY64_LITTLE 0x17
 #define CODE_CODEPOINTER 0x10
 #define CODE_INFIXPOINTER 0x11
-#define CODE_CUSTOM 0x12
+#define CODE_CUSTOM 0x12 /* deprecated */
+#define CODE_CUSTOM_LEN 0x18
+#define CODE_CUSTOM_FIXED 0x19
 
 #if ARCH_FLOAT_ENDIANNESS == 0x76543210
 #define CODE_DOUBLE_NATIVE CODE_DOUBLE_BIG
@@ -181,24 +183,10 @@ CAMLextern void caml_deserialize_block_2(void * data, intnat len);
 CAMLextern void caml_deserialize_block_4(void * data, intnat len);
 CAMLextern void caml_deserialize_block_8(void * data, intnat len);
 CAMLextern void caml_deserialize_block_float_8(void * data, intnat len);
-CAMLextern void caml_deserialize_error(char * msg);
 
-#ifdef CAML_INTERNALS
-
-/* Auxiliary stuff for sending code pointers */
-
-struct code_fragment {
-  char * code_start;
-  char * code_end;
-  unsigned char digest[16];
-  char digest_computed;
-};
-
-CAMLextern struct code_fragment * caml_extern_find_code(char *addr);
-
-extern struct ext_table caml_code_fragments_table;
-
-#endif /* CAML_INTERNALS */
+CAMLnoreturn_start
+CAMLextern void caml_deserialize_error(char * msg)
+CAMLnoreturn_end;
 
 #ifdef __cplusplus
 }
