@@ -160,9 +160,13 @@ and row_desc =
       row_more: type_expr;
       row_bound: unit; (* kept for compatibility *)
       row_closed: bool;
-      row_fixed: bool;
+      row_fixed: fixed_explanation option;
       row_name: (Path.t * type_expr list) option }
-
+and fixed_explanation =
+  | Univar of type_expr (** The row type was bound to an univar *)
+  | Fixed_private (** The row type is private *)
+  | Reified of Path.t (** The row was reified *)
+  | Rigid (** The row type was made rigid during constraint verification *)
 and row_field =
     Rpresent of type_expr option
   | Reither of bool * type_expr list * bool * row_field option ref
@@ -260,11 +264,6 @@ and value_kind =
                                         (* Self *)
   | Val_anc of (string * Ident.t) list * string
                                         (* Ancestor *)
-  | Val_unbound of value_unbound_reason (* Unbound variable *)
-
-and value_unbound_reason =
-  | Val_unbound_instance_variable
-  | Val_unbound_ghost_recursive
 
 (* Variance *)
 
@@ -300,7 +299,7 @@ type type_declaration =
     type_expansion_scope: int;
     type_loc: Location.t;
     type_attributes: Parsetree.attributes;
-    type_immediate: bool; (* true iff type should not be a pointer *)
+    type_immediate: Type_immediacy.t;
     type_unboxed: unboxed_status;
   }
 
@@ -413,9 +412,13 @@ type visibility =
 type module_type =
     Mty_ident of Path.t
   | Mty_signature of signature
-  | Mty_functor of Ident.t * module_type option * module_type
+  | Mty_functor of functor_parameter * module_type
   | Mty_alias of Path.t
 
+and functor_parameter =
+  | Unit
+  | Named of Ident.t option * module_type
+
 and module_presence =
   | Mp_present
   | Mp_absent
