@@ -36,14 +36,19 @@ val fprintf : out_channel -> ('a, out_channel, unit) format -> 'a
    The types and their meanings are:
 
    - [d], [i]: convert an integer argument to signed decimal.
+     The flag [#] adds underscores to large values for readability.
    - [u], [n], [l], [L], or [N]: convert an integer argument to
      unsigned decimal.  Warning: [n], [l], [L], and [N] are
      used for [scanf], and should not be used for [printf].
+     The flag [#] adds underscores to large values for readability.
    - [x]: convert an integer argument to unsigned hexadecimal,
      using lowercase letters.
+     The flag [#] adds a [0x] prefix to non zero values.
    - [X]: convert an integer argument to unsigned hexadecimal,
      using uppercase letters.
+     The flag [#] adds a [0X] prefix to non zero values.
    - [o]: convert an integer argument to unsigned octal.
+     The flag [#] adds a [0] prefix to non zero values.
    - [s]: insert a string argument.
    - [S]: convert a string argument to OCaml syntax (double quotes, escapes).
    - [c]: insert a character argument.
@@ -53,6 +58,7 @@ val fprintf : out_channel -> ('a, out_channel, unit) format -> 'a
      in the style [dddd.ddd].
    - [F]: convert a floating-point argument to OCaml syntax ([dddd.]
      or [dddd.ddd] or [d.ddd e+-dd]).
+     Converts to hexadecimal with the [#] flag (see [h]).
    - [e] or [E]: convert a floating-point argument to decimal notation,
      in the style [d.ddd e+-dd] (mantissa and exponent).
    - [g] or [G]: convert a floating-point argument to decimal notation,
@@ -101,8 +107,7 @@ val fprintf : out_channel -> ('a, out_channel, unit) format -> 'a
    - space: for signed numerical conversions, prefix number with a
      space if positive.
    - [#]: request an alternate formatting style for the integer types
-     ([x], [X], [o], [lx], [lX], [lo], [Lx], [LX], [Lo], [d], [i], [u],
-     [ld], [li], [lu], [Ld], [Li], [Lu], [nd], [ni], [nu]).
+     and the floating-point type [F].
 
    The optional [width] is an integer indicating the minimal
    width of the result. For instance, [%6d] prints an integer,
