@@ -19,20 +19,54 @@ type 'a t
 
 external create : int -> 'a t = "caml_weak_create"
 
+let create l =
+  if not (0 <= l && l <= Obj.Ephemeron.max_ephe_length) then
+    invalid_arg("Weak.create");
+  create l
+
 (** number of additional values in a weak pointer *)
 let additional_values = 2
 
 let length x = Obj.size(Obj.repr x) - additional_values
 
-external set : 'a t -> int -> 'a option -> unit = "caml_weak_set"
+let raise_if_invalid_offset e o msg =
+  if not (0 <= o && o < length e) then
+    invalid_arg(msg)
+
+external set' : 'a t -> int -> 'a -> unit = "caml_ephe_set_key"
+external unset : 'a t -> int -> unit = "caml_ephe_unset_key"
+let set e o x =
+  raise_if_invalid_offset e o "Weak.set";
+  match x with
+  | None -> unset e o
+  | Some x -> set' e o x
+
 external get : 'a t -> int -> 'a option = "caml_weak_get"
+let get e o =
+  raise_if_invalid_offset e o "Weak.get";
+  get e o
+
 external get_copy : 'a t -> int -> 'a option = "caml_weak_get_copy"
+let get_copy e o =
+  raise_if_invalid_offset e o "Weak.get_copy";
+  get_copy e o
+
 external check : 'a t -> int -> bool = "caml_weak_check"
+let check e o =
+  raise_if_invalid_offset e o "Weak.check";
+  check e o
+
 external blit : 'a t -> int -> 'a t -> int -> int -> unit = "caml_weak_blit"
+
 (* blit: src srcoff dst dstoff len *)
+let blit e1 o1 e2 o2 l =
+  if l < 0 || o1 < 0 || o1 > length e1 - l
+     || o2 < 0 || o2 > length e2 - l
+  then invalid_arg "Weak.blit"
+  else if l <> 0 then blit e1 o1 e2 o2 l
 
 let fill ar ofs len x =
-  if ofs < 0 || len < 0 || ofs + len > length ar
+  if ofs < 0 || len < 0 || ofs > length ar - len
   then raise (Invalid_argument "Weak.fill")
   else begin
     for i = ofs to (ofs + len - 1) do
@@ -52,6 +86,7 @@ module type S = sig
   val add : t -> data -> unit
   val remove : t -> data -> unit
   val find : t -> data -> data
+  val find_opt : t -> data -> data option
   val find_all : t -> data -> data list
   val mem : t -> data -> bool
   val iter : (data -> unit) -> t -> unit
@@ -165,8 +200,10 @@ module Make (H : Hashtbl.HashedType) : (S with type data = H.t) = struct
         t.table.(t.rover) <- emptybucket;
         t.hashes.(t.rover) <- [| |];
       end else begin
-        Obj.truncate (Obj.repr bucket) (prev_len + additional_values);
-        Obj.truncate (Obj.repr hbucket) prev_len;
+        let newbucket = weak_create prev_len in
+        blit bucket 0 newbucket 0 prev_len;
+        t.table.(t.rover) <- newbucket;
+        t.hashes.(t.rover) <- Array.sub hbucket 0 prev_len
       end;
       if len > t.limit && prev_len <= t.limit then t.oversize <- t.oversize - 1;
     end;
@@ -200,7 +237,9 @@ module Make (H : Hashtbl.HashedType) : (S with type data = H.t) = struct
     let sz = length bucket in
     let rec loop i =
       if i >= sz then begin
-        let newsz = min (3 * sz / 2 + 3) (Sys.max_array_length - 1) in
+        let newsz =
+          min (3 * sz / 2 + 3) (Sys.max_array_length - additional_values)
+        in
         if newsz <= sz then failwith "Weak.Make: hash bucket cannot grow more";
         let newbucket = weak_create newsz in
         let newhashes = Array.make newsz 0 in
@@ -255,7 +294,28 @@ module Make (H : Hashtbl.HashedType) : (S with type data = H.t) = struct
     find_or t d (fun h index -> add_aux t set (Some d) h index; d)
 
 
-  let find t d = find_or t d (fun h index -> raise Not_found)
+  let find t d = find_or t d (fun _h _index -> raise Not_found)
+
+  let find_opt t d =
+    let h = H.hash d in
+    let index = get_index t h in
+    let bucket = t.table.(index) in
+    let hashes = t.hashes.(index) in
+    let sz = length bucket in
+    let rec loop i =
+      if i >= sz then None
+      else if h = hashes.(i) then begin
+        match get_copy bucket i with
+        | Some v when H.equal v d
+           -> begin match get bucket i with
+              | Some _ as v -> v
+              | None -> loop (i + 1)
+              end
+        | _ -> loop (i + 1)
+      end else loop (i + 1)
+    in
+    loop 0
+
 
   let find_shadow t d iffound ifnotfound =
     let h = H.hash d in
@@ -276,7 +336,9 @@ module Make (H : Hashtbl.HashedType) : (S with type data = H.t) = struct
 
   let remove t d = find_shadow t d (fun w i -> set w i None) ()
 
-  let mem t d = find_shadow t d (fun w i -> true) false
+
+  let mem t d = find_shadow t d (fun _w _i -> true) false
+
 
   let find_all t d =
     let h = H.hash d in
